
remove(list=ls())
library(maxLik)
library(matrixcalc)
library(car)
library(hmmm)
source("programCUSCUB.R") 

data<-read.table(file="civicdata.txt", sep = "\t",header=TRUE,
colClasses=NA,blank.lines.skip=FALSE)

names(data)
str(data)

##### factor variables

#factor geographical area 
data$geoarea<-factor(data$geoarea, levels=c("1","2","3"),labels=c("N","C","S"))
data$geoarea<-C(data$geoarea,"contr.SAS") # the last category is used as reference
# orthogonal polynomials for no equal spaced levels
data$edu<-as.factor(data$education)
x<-as.numeric(levels(data$edu))
Co<-contr.poly(8,x) 
g<-2 
#1 for linear effect, 2 for quadratic effect, etc
data$education<-C(data$edu,Co,g) #factor with g polynomial contrasts 

############  models #################

# covariates for feeling parameters csi 

f.formula=list(
ticket ~ gender + data$geoarea + data$education,
money ~ gender + data$geoarea +  data$education,
car ~ gender + data$geoarea + data$education
)

# no covariates for the uncertainty parameter pi

fu.formula=list(
ticket ~ 1,
money ~ 1,
car ~ 1
)

################## ESTIMATION of the parameters ##################

# model CUSCUB
modFF<-MS2.Cub(data, f.formula, fu.formula, finalHessian = TRUE, 
                 iterlim = 1500, method = "BHHH", numeric = FALSE, epseps = 1e-12,
                 CUSB = FALSE, CUSCUB = FALSE
                          )
# model CUSB
modTF<-MS2.Cub(data, f.formula, fu.formula, finalHessian = "BHHH",
                 iterlim = 1500, method = "BHHH", numeric = FALSE, epseps = 1e-12,
                 CUSB = TRUE, CUSCUB = FALSE
                 )

# model SCUB
modFT<-MS2.Cub(data, f.formula, fu.formula, print.level=2, finalHessian = TRUE,
                 iterlim = 1500, method = "BHHH", numeric = FALSE, epseps = 1e-12,
                 CUSB = FALSE, CUSCUB = TRUE
                )

# model SSB
modTT<-MS2.Cub(data, f.formula, fu.formula, print.level = 2, finalHessian = "BHHH",
                 iterlim = 1500, method = "BHHH", numeric = FALSE, epseps = 1e-12,
                 CUSB = TRUE, CUSCUB = TRUE
                 )

# MLE, s.e. and test of the parameters of the models
summary(modFF$fit) # the best model
summary(modTF$fit)
summary(modFT$fit)
summary(modTT$fit)

# BIC values
AIC(modFF$fit,k=log(dim(data)[2])) 
AIC(modTF$fit,k=log(dim(data)[2]))
AIC(modFT$fit,k=log(dim(data)[2]))
AIC(modTT$fit,k=log(dim(data)[2]))

# model CUSCUB under the homogeneous association hypothesis
modFFass<-MS2.Cub(data, f.formula, fu.formula, print.level = 2, finalHessian = TRUE,
                 iterlim = 1500, method = "BHHH", numeric = FALSE, epseps = 1e-12,
                 CUSB = FALSE, CUSCUB = FALSE, WMAT = matrix(1,3))

############################ Pearson Residuals ##################################

rr<-MS2.CUB.UTIL(cuscub = modFF, data = data, FISHER = FALSE) 
rr$CV # range of omega parameters
rr$STAT # descriptive statistics of Pearson Residuals 

# box plot of Pearson Residuals
d<-rr$RES # Pearson Residuals
boxplot(d$rePearson[d$f>=0&abs(d$rePearson)<5],  ylab = "Pearson residuals")
title("Box plot of residuals (not exceeding 5 in abs)")

# Figure 1
f<-function(x){max(as.numeric(x))-min(as.numeric(x))}
d$tot<-apply(d[,1:3],1,f)
boxplot(d$rePearson[d$f>=0]~d$tot[d$f>=0], xlab = "max distance among answers", ylab = "Pearson residuals")
title("Box plot of residuals (wrt the distance among answers)")   
